@@ -5,12 +5,16 @@
 */
 
 #include "quickengine.h"
+#include "quicksettings.h"
 
+#if KNEWSTUFFQUICK_BUILD_DEPRECATED_SINCE(5, 81)
 #include <KAuthorized>
+#endif
 #include <KLocalizedString>
 
 #include "categoriesmodel.h"
 #include "quickquestionlistener.h"
+#include "searchpresetmodel.h"
 
 #include "engine.h"
 
@@ -20,18 +24,23 @@ public:
     Private()
         : engine(nullptr)
         , categoriesModel(nullptr)
-    {}
+        , searchPresetModel(nullptr)
+    {
+    }
     KNSCore::Engine *engine;
     bool isLoading{false};
+    bool isValid{false};
     CategoriesModel *categoriesModel;
+    SearchPresetModel *searchPresetModel;
     QString configFile;
 
+#if KNEWSTUFF_BUILD_DEPRECATED_SINCE(5, 82)
     KNSCore::EntryInternal::List changedEntries;
-    static KNSCore::EntryWrapper *getChangedEntry(QQmlListProperty<KNSCore::EntryWrapper>* property, int i)
+    static KNSCore::EntryWrapper *getChangedEntry(QQmlListProperty<KNSCore::EntryWrapper> *property, int i)
     {
         KNSCore::EntryWrapper *entry{nullptr};
         if (property) {
-            Private* d = static_cast<Engine::Private*>(property->data);
+            Private *d = static_cast<Engine::Private *>(property->data);
             if (d) {
                 if (i >= 0 && i < d->changedEntries.count()) {
                     // Lifetime management for these objects should be done by the consumer,
@@ -42,17 +51,18 @@ public:
         }
         return entry;
     }
-    static int getChangedEntriesCount(QQmlListProperty<KNSCore::EntryWrapper>* property)
+    static int getChangedEntriesCount(QQmlListProperty<KNSCore::EntryWrapper> *property)
     {
         int count{0};
         if (property) {
-            Private* d = static_cast<Engine::Private*>(property->data);
+            Private *d = static_cast<Engine::Private *>(property->data);
             if (d) {
                 count = d->changedEntries.count();
             }
         }
         return count;
     }
+#endif
 };
 
 Engine::Engine(QObject *parent)
@@ -66,10 +76,12 @@ Engine::~Engine()
     delete d;
 }
 
+#if KNEWSTUFFQUICK_BUILD_DEPRECATED_SINCE(5, 81)
 bool Engine::allowedByKiosk() const
 {
-    return KAuthorized::authorize(QStringLiteral("ghns"));
+    return KAuthorized::authorize(KAuthorized::GHNS);
 }
+#endif
 
 QString Engine::configFile() const
 {
@@ -84,10 +96,10 @@ void Engine::setConfigFile(const QString &newFile)
         d->configFile = newFile;
         Q_EMIT configFileChanged();
 
-        if (allowedByKiosk()) {
+        if (KNewStuffQuick::Settings::instance()->allowedByKiosk()) {
             if (!d->engine) {
                 d->engine = new KNSCore::Engine(this);
-                connect(d->engine, &KNSCore::Engine::signalProvidersLoaded, this, [=](){
+                connect(d->engine, &KNSCore::Engine::signalProvidersLoaded, this, [=]() {
                     d->isLoading = false;
                     Q_EMIT isLoadingChanged();
                 });
@@ -99,31 +111,51 @@ void Engine::setConfigFile(const QString &newFile)
                         busyMessage(d->engine->busyMessage());
                     }
                 });
-                connect(d->engine, &KNSCore::Engine::signalErrorCode, this, [=](const KNSCore::ErrorCode &errorCode, const QString &message, const QVariant &/*metadata*/) {
-                    if (errorCode == KNSCore::ProviderError) {
-                        // This means loading the providers file failed entirely and we cannot complete the
-                        // initialisation. It also means the engine is done loading, but that nothing will
-                        // work, and we need to inform the user of this.
-                        d->isLoading = false;
-                        Q_EMIT isLoadingChanged();
-                    }
-                    Q_EMIT errorMessage(message);
-                });
-                connect(d->engine, &KNSCore::Engine::signalEntryEvent,
-                        this, [this](const KNSCore::EntryInternal &entry, KNSCore::EntryInternal::EntryEvent event) {
+                connect(d->engine,
+                        &KNSCore::Engine::signalErrorCode,
+                        this,
+                        [=](const KNSCore::ErrorCode &theErrorCode, const QString &message, const QVariant &metadata) {
+                            Q_EMIT errorCode(static_cast<ErrorCode>(theErrorCode), message, metadata);
+                            if (theErrorCode == KNSCore::ProviderError) {
+                                // This means loading the providers file failed entirely and we cannot complete the
+                                // initialisation. It also means the engine is done loading, but that nothing will
+                                // work, and we need to inform the user of this.
+                                d->isLoading = false;
+                                Q_EMIT isLoadingChanged();
+                            }
+                            Q_EMIT errorMessage(message);
+                        });
+                connect(d->engine,
+                        &KNSCore::Engine::signalEntryEvent,
+                        this,
+                        [this](const KNSCore::EntryInternal &entry, KNSCore::EntryInternal::EntryEvent event) {
+                            KNSCore::EntryWrapper *wrappedEntry = new KNSCore::EntryWrapper(entry, this);
+                            // Just forward the event but not do anything more
                             if (event != KNSCore::EntryInternal::StatusChangedEvent) {
+                                Q_EMIT entryEvent(wrappedEntry, (EntryEvent)event);
                                 return;
                             }
+
+                            // We do not want to emit the entries changed signal for intermediate changed
+                            // this would cause the KCMs to reload their view unnecessarily, BUG: 431568
+                            if (entry.status() == KNS3::Entry::Installing || entry.status() == KNS3::Entry::Updating) {
+                                return;
+                            }
+                            Q_EMIT entryEvent(wrappedEntry, (EntryEvent)event);
+#if KNEWSTUFF_BUILD_DEPRECATED_SINCE(5, 82)
                             if (d->changedEntries.contains(entry)) {
                                 d->changedEntries.removeAll(entry);
                             }
                             d->changedEntries << entry;
                             Q_EMIT changedEntriesChanged();
+#endif
                         });
                 Q_EMIT engineChanged();
                 KNewStuffQuick::QuickQuestionListener::instance();
                 d->categoriesModel = new CategoriesModel(this);
                 Q_EMIT categoriesChanged();
+                d->searchPresetModel = new SearchPresetModel(this);
+                Q_EMIT searchPresetModelChanged();
                 // And finally, let's just make sure we don't miss out the various things here getting changed
                 // In other words, when we're asked to reset the view, actually do that
                 connect(d->engine, &KNSCore::Engine::signalResetView, this, &Engine::categoriesFilterChanged);
@@ -135,12 +167,14 @@ void Engine::setConfigFile(const QString &newFile)
                 Q_EMIT sortOrderChanged();
                 Q_EMIT searchTermChanged();
             }
-            d->engine->init(d->configFile);
+            d->isValid = d->engine->init(d->configFile);
             Q_EMIT engineInitialized();
         } else {
             // This is not an error message in the proper sense, and the message is not intended to look like an error (as there is really
             // nothing the user can do to fix it, and we just tell them so they're not wondering what's wrong)
-            Q_EMIT message(i18nc("An informational message which is shown to inform the user they are not authorized to use GetHotNewStuff functionality", "You are not authorized to Get Hot New Stuff. If you think this is in error, please contact the person in charge of your permissions."));
+            Q_EMIT message(
+                i18nc("An informational message which is shown to inform the user they are not authorized to use GetHotNewStuff functionality",
+                      "You are not authorized to Get Hot New Stuff. If you think this is in error, please contact the person in charge of your permissions."));
         }
     }
 }
@@ -247,17 +281,23 @@ QString Engine::searchTerm() const
 
 void Engine::setSearchTerm(const QString &newSearchTerm)
 {
-    if (d->engine && d->engine->searchTerm() != newSearchTerm) {
+    if (d->engine && d->isValid && d->engine->searchTerm() != newSearchTerm) {
         d->engine->setSearchTerm(newSearchTerm);
         Q_EMIT searchTermChanged();
     }
 }
 
+QObject *Engine::searchPresetModel() const
+{
+    return d->searchPresetModel;
+}
+
 void Engine::resetSearchTerm()
 {
     setSearchTerm(QString{});
 }
 
+#if KNEWSTUFF_BUILD_DEPRECATED_SINCE(5, 82)
 QQmlListProperty<KNSCore::EntryWrapper> Engine::changedEntries()
 {
     return QQmlListProperty<KNSCore::EntryWrapper>(this, d, &Private::getChangedEntriesCount, &Private::getChangedEntry);
@@ -270,6 +310,14 @@ int Engine::changedEntriesCount() const
 
 void Engine::resetChangedEntries()
 {
-    d->changedEntries.clear();
-    Q_EMIT changedEntriesChanged();
+    if (!d->changedEntries.isEmpty()) {
+        d->changedEntries.clear();
+        Q_EMIT changedEntriesChanged();
+    }
+}
+#endif
+
+bool Engine::isValid()
+{
+    return d->isValid;
 }
